/*
 * 03/26/2002 - 21:14:14
 * 
 * MPD - Copyright (C) 2002 Dreux Loic zgoblin@linuxfreemail.com
 * 
 *  Modifications 
 *  -------------
 *  Date : 2009 janvier 22
 *  @auteur : Bruno Dabo <bruno.dabo@lywoon.com>
 *   
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.analyse.merise.mcd.composant;

import java.awt.Color;
import java.awt.Dimension;
import java.util.Enumeration;
import java.util.Hashtable;

import org.analyse.core.gui.zgraph.ZGraphique;
import org.analyse.core.util.Utilities;
import org.analyse.merise.gui.table.DictionnaireTable;
import org.analyse.merise.sql.SQLCommand;
import org.analyse.core.util.Constantes ;

public class MPDComponent extends ZGraphique
{
    private double width;

    private Hashtable foreignKeys;

    public MPDComponent()
    {
        super();

        foreignKeys = new Hashtable();

        setEnabled(false);
        setBackground(Color.white);
    }

    /**
     * @return la taille du composant
     */
    public Dimension getPreferredSize()
    {
        int x = 0, y = 0;
        for (Enumeration e = enumElements(); e.hasMoreElements();) {
            MPDEntite o = (MPDEntite) e.nextElement();
            x = o.getX() + o.getWidth() > x ? o.getX() + o.getWidth() : x;
            y = o.getY() + o.getHeight() > y ? o.getY() + o.getHeight() : y;
        }
        return new Dimension(x + 30, y + 30);
    }

    /**
     * @return un MPDEntite
     * @param nom
     *            Nom de l'entité à retourner
     */
    public MPDEntite getMPDEntite(String name)
    {
        for (Enumeration e = enumElements(); e.hasMoreElements();) {
            MPDEntite o = (MPDEntite) e.nextElement();
            if (o.getName().equals(name))
                return o;
        }
        return null;
    }

    public MPDEntite getMPDEntite(int i)
    {
        return (MPDEntite) (getElement(i));
    }

    /**
     * Modifie la largeur du <code>MPDComponent</code>
     */
    public void setWidth(double width)
    {
        this.width = width;
    }

    /**
     * Ajoute une entité au vecteur.
     */
    public void addMPDEntite(MPDEntite ent)
    {
        addElement(ent);

        repaint();
    }

    public void addForeignKey(String cleEtrangere, String table)
    {
        foreignKeys.put(cleEtrangere, table);
    }

    public String toString()
    {
        String res = "{\n";
        for (Enumeration e = enumElements(); e.hasMoreElements();) {
            res += e.nextElement() + "\n";
        }
        return "}" + res;
    }

    /**
     * Construit les requetes SQL.
     */
    public void buildSQL(DictionnaireTable data, SQLCommand sql)
    {
        String text, info;
        MPDEntite ent;
        int cmp, nbId;
        boolean isFirst;

        sql.clear();

        for (Enumeration e = enumElements(); e.hasMoreElements();) {
            ent = (MPDEntite) (e.nextElement());
            text = "DROP TABLE IF EXISTS " + Utilities.normaliseString(ent.getName(), Constantes.LOWER) + " ;" ;
            sql.addRequest(text);
            
            text = "CREATE TABLE " + Utilities.normaliseString(ent.getName(), Constantes.LOWER) + " (";
            cmp = 0;
            nbId = ent.sizeIdentifiant();
            
            /*
             * bruno
             * date : mars 2009
             * je rajoute une surcouche pour la gestion des relations reflexives. 
             * Code à améliorer progressivement lors de la refonte  
             */
            Boolean premierefois = true ;
            String defautType = "" ;
            String defautSize = "" ;
            String type = ""; 
            
            for (Enumeration e2 = ent.elementsInformations(); e2.hasMoreElements(); cmp++) {
            	
            	info = (String) (e2.nextElement());
            	
            	if ( premierefois ) {  // ajout des relations reflexives
            		premierefois = false ; 
            		defautType = ( String ) data.getValue(info, DictionnaireTable.TYPE) ; 
            		
            		if (((Integer) data.getValue(info, DictionnaireTable.SIZE)).intValue() != 0
                			&& !sql.getTypesWithoutSize().contains(data.getValue(info, DictionnaireTable.TYPE)))
                		defautSize = ( String ) data.getValue(info, DictionnaireTable.SIZE) ;
            	}
            	
                try {
                	type = ( String ) data.getValue(info, DictionnaireTable.TYPE) ; 
                } catch ( Exception ex) {
                	type = defautType ; 
                }
                
                text += info.toLowerCase() + " " + type ;

                try {
                	if (((Integer) data.getValue(info, DictionnaireTable.SIZE)).intValue() != 0
                			&& !sql.getTypesWithoutSize().contains(data.getValue(info, DictionnaireTable.TYPE)))
                		text += "(" + data.getValue(info, DictionnaireTable.SIZE) + ")";
                } catch ( Exception ex ) {
                	
                		if ( ! defautSize.equals(""))
                			text += "(" + defautSize + ")";
                }
                
                if (cmp < nbId)
                    text += " NOT NULL";
                else {
                	if ( ! ent.isForeignKeyCanBeNull() ) {
                		
                		if ( ent.getForeignKey( info ) != null )    // on a affaire à une clé étrangère
                			text += " NOT NULL";
                	}
                }

                if (e2.hasMoreElements())
                    text += ", ";
            }
            text += ");";

            sql.addRequest(text);
        }

        for (Enumeration e = enumElements(); e.hasMoreElements();) {
            ent = (MPDEntite) (e.nextElement());
            nbId = ent.sizeIdentifiant();
            //liaison reflexive
            //  if ( ent.informations.size() != nbId )
            //	  continue ; 
              
            text = "ALTER TABLE "
                    + Utilities.normaliseString(ent.getName(), Constantes.LOWER)
                    + " ADD CONSTRAINT PK_"
                    + Utilities.normaliseString(ent.getName(), Constantes.LOWER)
                    + " PRIMARY KEY (";
            cmp = 0;
            nbId = ent.sizeIdentifiant();

            for (Enumeration e2 = ent.elementsInformations(); e2
                    .hasMoreElements()
                    && cmp < nbId;) {
                text += (String) (e2.nextElement());
                cmp++;
                if (cmp < nbId)
                    text += ", ";
            }
            text += ");";

            sql.addRequest(text);
        }

        
        for (Enumeration e = enumElements(); e.hasMoreElements();) {
            ent = (MPDEntite) (e.nextElement());
        
            if (!ent.foreignKeysIsEmpty()) {
                for (Enumeration e2 = ent.elementsInformations(); e2
                        .hasMoreElements();) {
                    info = (String) (e2.nextElement());
                    if (ent.getTableForeignKey(info) != null) {
                        text = "ALTER TABLE "
                            + Utilities.normaliseString(ent.getName(),
                            		Constantes.LOWER) + " ADD CONSTRAINT FK_"
                                + Utilities.normaliseString(ent.getName(),
                                		Constantes.LOWER) + "_" + info
                                + " FOREIGN KEY (" + info + ") REFERENCES "
                                + ent.getTableForeignKey(info) + " (" + info
                                + ");";
                        sql.addRequest(text);
                    }              
                }                
            }
        }
    }

    /**
     *  
     */
    public void clear()
    {
        foreignKeys.clear();
        super.clearAll();
    }
}
